/**********
This program is free software: you can redistribute it and/or modify
it under the terms of the GNU General Public License as published by
the Free Software Foundation, either version 3 of the License, or
(at your option) any later version.

This program is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
GNU General Public License for more details.

You should have received a copy of the GNU General Public License
along with this program.  If not, see <http://www.gnu.org/licenses/>.
**********/
/*
    A C++ program to parse DJI's ".txt" log files (recorded by the "DJI Go 4" app).
    Version 2021-01-19

    Copyright (c) 2021 Live Networks, Inc.  All rights reserved.
    For the latest version of this program (and more information), visit http://djilogs.live555.com

    Parsing COMPONENT records within DJI ".txt" files.
    Implementation.
*/

#include "RecordAndDetailsParser.hh"

void RecordAndDetailsParser::parseRecord_COMPONENT(u_int8_t const*& ptr, u_int8_t const* limit) {
  // Because these records always seem to appear at the beginning of the file, before the
  // first OSD record, treat each (known) component type as a separate column in the output:

  // COMPONENT.type: 2 bytes little-endian unsigned
  u_int16_t componentType = get2BytesLE(ptr, limit);

  // Length of the following serial number: 1 byte
  unsigned serialNumberLength = getByte(ptr, limit);

  // COMPONENT.<type>Sn: string (length "serialNumberLength"):
  noteStringField(componentType == 1 ? "COMPONENT.cameraSn":
		  componentType == 2 ? "COMPONENT.aircraftSn":
		  componentType == 3 ? "COMPONENT.rcSn":
		  componentType == 4 ? "COMPONENT.batterySn":
		  "COMPONENT.unknownSn",
		  ptr, serialNumberLength, limit);
}
